@@ -1,3 +1,24 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #include "ant.h"
 #include "graphics.h"
 #include "levelplayer.h"
@@ -77,7 +98,7 @@ typedef enum {
 
 
 
-// init the ant state for level entering
+// initialize the ant state for level entering
 // the level is saved and used later on for dirty block
 // marking, and level modification
 ant_c::ant_c(levelPlayer_c & level, graphics_c & gr, surface_c & target): level(level), gr(gr), vid(target) {
@@ -99,6 +120,7 @@ void ant_c::initForLevel(void) {
   fallingHight = 0;
   direction = 1;
   pushDelay = 0;
+  downChecker = false;
 
   finalAnimationPlayed = levelFail = levelSuccess = false;
 }
@@ -128,11 +150,17 @@ void ant_c::draw(void) {
           img = 7;
         }
 
-        vid.blit(gr.getDomino((carriedDomino - 1) % 10, img), x, y);
+        if (carriedDomino > gr.numDominoTypes)
+          vid.blit(gr.getDomino(0, img), x, y);
+        else
+          vid.blit(gr.getDomino(carriedDomino-1, img), x, y);
       }
       else
       {
-        vid.blit(gr.getCarriedDomino(img-32, (carriedDomino-1)%10), x, y);
+        if (carriedDomino > gr.numDominoTypes)
+          vid.blit(gr.getCarriedDomino(img-32, 0), x, y);
+        else
+          vid.blit(gr.getCarriedDomino(img-32, carriedDomino-1), x, y);
       }
     }
     if (animation >= AntAnimCarryLeft && animation <= AntAnimCarryStopRight)
@@ -526,6 +554,9 @@ unsigned int ant_c::SFPushLeft(void) {
 
         switch(level.getDominoType(blockX-1, blockY)) {
           case levelData_c::DominoTypeStopper:
+          case levelData_c::DominoTypeCounter1:
+          case levelData_c::DominoTypeCounter2:
+          case levelData_c::DominoTypeCounter3:
             pushDelay = 5;
             pushAnimation = AntAnimPushStopperLeft;
             break;
@@ -571,6 +602,9 @@ unsigned int ant_c::SFPushRight(void) {
 
         switch(level.getDominoType(blockX+1, blockY)) {
           case levelData_c::DominoTypeStopper:
+          case levelData_c::DominoTypeCounter1:
+          case levelData_c::DominoTypeCounter2:
+          case levelData_c::DominoTypeCounter3:
             pushDelay = 5;
             pushAnimation = AntAnimPushStopperRight;
             break;
@@ -1215,10 +1249,6 @@ unsigned int ant_c::checkForNoKeyActions(void) {
   return ReturnAntState;
 }
 
-static void SaveLevelState(void) {
-  // TODO: we need to fill this function
-}
-
 bool ant_c::CanPlaceDomino(int x, int y, int ofs) {
 
   if (ofs < -1 || ofs > 1) return false;
@@ -1300,7 +1330,7 @@ unsigned int ant_c::SFNextAction(void) {
   bool onLadder = (animation >= AntAnimLadder1 && animation <= AntAnimLadder4) ||
                   (animation >= AntAnimCarryLadder1 && animation <= AntAnimCarryLadder4);
 
-  // when we hav no ground below us and are not on a ladder we need
+  // when we have no ground below us and are not on a ladder we need
   // to fall down
   if (level.noGround(blockX, blockY, onLadder))
   {
@@ -1396,7 +1426,6 @@ unsigned int ant_c::SFNextAction(void) {
       }
       else
       {
-        SaveLevelState();
         numPushsLeft--;
         returnState = AntAnimPushLeft;
         direction = -1;
@@ -1407,7 +1436,6 @@ unsigned int ant_c::SFNextAction(void) {
       downChecker = true;
       if (numPushsLeft && keyMask & KEY_ACTION)
       {
-        SaveLevelState();
         blockX++;
         animation = returnState = AntAnimPushLeft;
         if (!PushableDomino(blockX, blockY, -1))
@@ -1507,7 +1535,6 @@ unsigned int ant_c::SFNextAction(void) {
       }
       else
       {
-        SaveLevelState();
         numPushsLeft--;
         returnState = AntAnimPushRight;
         direction = 1;
@@ -1518,7 +1545,6 @@ unsigned int ant_c::SFNextAction(void) {
       downChecker = true;
       if (numPushsLeft != 0 && keyMask & KEY_ACTION)
       {
-        SaveLevelState();
         blockX--;
         animation = returnState = AntAnimPushRight;
         if (!PushableDomino(blockX, blockY, 1))
