@@ -1,3 +1,24 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #include "leveldata.h"
 
 #include "textsections.h"
@@ -41,13 +62,18 @@ const std::string levelData_c::dominoChars =
   ":" /* DominoTypeVanish   */
   "!" /* DominoTypeTrigger  */
   "A" /* DominoTypeAscender */
+  "X" /* DominoTypeConnectedA */
+  "x" /* DominoTypeConnectedA */
+  "a" /* DominoTypeCounter1 */
+  "b" /* DominoTypeCounter1 */
+  "c" /* DominoTypeCounter1 */
   ;
 
 bool levelData_c::isDominoChar(char ch) {
   return dominoChars.find_first_of(ch) != std::string::npos;
 }
 
-void levelData_c::load(const textsections_c & sections) {
+void levelData_c::load(const textsections_c & sections, const std::string & userString) {
 
   memset(level, 0, sizeof(level));
 
@@ -243,8 +269,17 @@ void levelData_c::load(const textsections_c & sections) {
       timeLeftStream << timeLeft;
       sha1.update(timeLeftStream.str());
     }
+    sha1.update(userString);
     checksum = sha1.final();
   }
+  {
+    SHA1 sha1;
+    for (unsigned int y = 0; y < 13; y++) {
+      sha1.update(levelRows[y]);
+    }
+    sha1.update(userString);
+    checksumNoTime = sha1.final();
+  }
 }
 
 void levelData_c::save(std::ostream & stream) const {
@@ -468,40 +503,26 @@ bool levelData_c::levelCompleted(int & fail) const {
         }
         else if (getDominoType(x, y) == DominoTypeTumbler)
         { // tumbler must lie on something or lean against a wall
-          // not falln far enough
-          if (getDominoState(x, y) > 3 && getDominoState(x, y) < 13) {
+          // not fallen far enough
+          if (getDominoState(x, y) >= 3 && getDominoState(x, y) <= 13) {
             fail = 6;
             return false;
           }
 
-          // check if we lean against a step
-          if (getDominoState(x, y) == 3 &&
-              getFg(x-1, y) != FgElementPlatformStep4 &&
-              getFg(x-1, y) != FgElementPlatformStep7)
-          {
-            fail = 6;
-            return false;
-          }
-
-          if (getDominoState(x, y) == 13 &&
-              getFg(x+1, y) != FgElementPlatformStep4 &&
-              getFg(x+1, y) != FgElementPlatformStep7)
-          {
-            fail = 6;
-            return false;
-          }
-
-          // falln far enough but neighbor empty
+          // fallen far enough but neighbor empty
           if (   getDominoState(x, y) <= 2
-              && getDominoType(x-1, y) == DominoTypeEmpty
-              && (getDominoType(x-2, y) == DominoTypeEmpty || getDominoState(x-2, y) < 14))
+              && ((x < 1) || (getDominoType(x-1, y) == DominoTypeEmpty))
+              && ((x < 2) || (getDominoType(x-2, y) == DominoTypeEmpty || getDominoState(x-2, y) < 14))
+             )
           {
             fail = 6;
             return false;
           }
           if (   getDominoState(x, y) >= 14
-              && getDominoType(x+1, y) == DominoTypeEmpty
-              && (getDominoType(x+2, y) == DominoTypeEmpty || getDominoState(x+2, y) > 2)) {
+              && ((x > 18) || (getDominoType(x+1, y) == DominoTypeEmpty))
+              && ((x > 17) || (getDominoType(x+2, y) == DominoTypeEmpty || getDominoState(x+2, y) > 2))
+             )
+          {
             fail = 6;
             return false;
           }
@@ -522,7 +543,6 @@ bool levelData_c::levelCompleted(int & fail) const {
           }
           // in this case we might still succeed, when we lean against a block
           if (   getDominoState(x, y) == 3
-              && getFg(x-1, y) != FgElementPlatformStep4
               && (   getDominoType(x-1, y) != DominoTypeStopper
                   || getDominoType(x+1, y) == DominoTypeEmpty
                   || (getDominoDir(x+1, y) != -1 && getDominoType(x+1, y) != DominoTypeSplitter))
@@ -534,7 +554,6 @@ bool levelData_c::levelCompleted(int & fail) const {
           }
 
           if (   getDominoState(x, y) == 13
-              && getFg(x+1, y) != FgElementPlatformStep7
               && (   getDominoType(x+1, y) != DominoTypeStopper
                   || getDominoType(x-1, y) == DominoTypeEmpty
                   || (getDominoDir(x-1, y) != 1 && getDominoType(x-1, y) != DominoTypeSplitter))
