@@ -1,3 +1,24 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #include "graphicsn.h"
 #include "textsections.h"
 #include "levelplayer.h"
@@ -21,9 +42,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <libintl.h>
 
-#include <libosso.h>
+#if defined(__AROS__)
+#include <proto/dos.h>
+#endif
 
 static void check_record(const std::string & rec_path, levelsetList_c & levelsetList,
                          unsigned int & count, unsigned int & failed,
@@ -34,7 +56,7 @@ static void check_record(const std::string & rec_path, levelsetList_c & levelset
   graphicsN_c gr("");
   surface_c surf;
   levelPlayer_c l(surf, gr);
-  levelsetList.getLevelset(rec.getLevelsetName()).loadLevel(l, rec.getLevelName());
+  levelsetList.getLevelset(rec.getLevelsetName()).loadLevel(l, rec.getLevelName(), "");
   ant_c a(l, gr, surf);
 
   while (!rec.endOfRecord()) {
@@ -61,7 +83,7 @@ static void check_record(const std::string & rec_path, levelsetList_c & levelset
 
 void check(int argn, char * argv[], std::string checker(const ant_c & a, const levelData_c & l)) {
   levelsetList_c levelsetList;
-  levelsetList.load("levels");
+  levelsetList.load("levels", "");
 
   unsigned int count = 0;
   unsigned int failed = 0;
@@ -124,14 +146,31 @@ std::string checker3(const ant_c & a, const levelData_c & l) {
 
 static std::string getDataDir(void)
 {
+#if defined (__AROS__)
+  static char binaryPath[1024] = {0};
+  if (!*binaryPath)
+    NameFromLock(GetProgramDir(), binaryPath, sizeof(binaryPath));
+  return std::string(binaryPath);
+#else
   struct stat st;
   return std::string((stat(PKGDATADIR, &st) == 0) ? PKGDATADIR : ".");
+#endif
 }
 
 static std::string getLocaleDir(void)
 {
+#if defined (__AROS__)
+  static char localePath[1024] = {0};
+  if (!*localePath)
+  {
+    AddPart(localePath, getDataDir().c_str(), sizeof(localePath));
+    AddPart(localePath, "locale", sizeof(localePath));
+  }
+  return localePath;
+#else
   struct stat st;
   return std::string((stat(LOCALEDIR, &st) == 0) ? LOCALEDIR : "locale");
+#endif
 }
 
 static unsigned int getKeyMask(void) {
@@ -143,14 +182,14 @@ static unsigned int getKeyMask(void) {
   if ( keystate[SDLK_DOWN] ) keyMask |= KEY_DOWN;
   if ( keystate[SDLK_LEFT] ) keyMask |= KEY_LEFT;
   if ( keystate[SDLK_RIGHT] ) keyMask |= KEY_RIGHT;
-  if ( keystate[SDLK_LSHIFT] ) keyMask |= KEY_ACTION;
+  if ( keystate[SDLK_SPACE] ) keyMask |= KEY_ACTION;
 
   return keyMask;
 }
 
 // make a play tick
 // return codes:
-// 0 nothing happend
+// 0 nothing happened
 // 1 success
 // 2 too slow
 // 3 crashes
@@ -188,10 +227,14 @@ int playTick(levelPlayer_c & l, ant_c & a)
 
 // the states of the program
 typedef enum {
-  ST_INIT,     // inistal transition state
+  ST_INIT,     // initial transition state
   ST_MAIN,     // main menu
-  ST_CONFIG,   // config menu
-  ST_CONFIGTOG,// config menu toggled something, repaint
+  ST_PROFILE,  // state for profile selection
+  ST_PROFILE_INIT, // profile selection at startup
+  ST_PROFILE_IN, // profile name input
+  ST_PROFILE_DEL, // select profile to delete
+  ST_LEVELCONF,// configure while playing
+  ST_CONFIG,   // configure menu
   ST_LEVELSET, // level set selection
   ST_LEVEL,    // level selection
   ST_PREREPLAY,// prepare replay for running
@@ -200,6 +243,7 @@ typedef enum {
   ST_PLAY,     // currently playing
   ST_FAILDELAY,// play a second more after failing
   ST_SOLVED,   // current level solved
+  ST_TIMEOUT,  // took longer than level time
   ST_FAILED,   // current level failed (reason in failreason)
   ST_HELP,     // help dialog showing
   ST_QUIT,     // play exit dialog showing
@@ -207,17 +251,24 @@ typedef enum {
   ST_ABOUT,
 } states_e;
 
+static levelsetList_c * loadAllLevels(const std::string & datadir, const std::string & userString)
+{
+  levelsetList_c * levelsetList = new levelsetList_c();
+
+  levelsetList->load(datadir + "/levels", userString);
+  {
+    const std::string userleveldir(getHome() + "/.pushover/levels");
+    struct stat st;
+    if (stat(userleveldir.c_str(), &st) == 0)
+      levelsetList->load(userleveldir, userString);
+  }
+
+  return levelsetList;
+}
+
 
 int main(int argc, char * argv[]) {
 
-  osso_context_t *osso_context = NULL;
-  putenv("SDL_VIDEO_X11_WMCLASS=pushover");
-  
-  osso_context = osso_initialize ("pushover", "0.0.2", 0, NULL);
-  if(osso_context == NULL) {
-    fprintf(stderr, "error initiating osso context\n");
-  }
-
   // filter out the no graphic cases, they are special and will be treated
   // separately
   if (argc >= 3 && strcmp(argv[1], "-c") == 0)   // the must complete tests
@@ -232,13 +283,19 @@ int main(int argc, char * argv[]) {
     return 0;
   }
 
-  if (argc >= 3 && strcmp(argv[1], "-f") == 0)   // the must crash tests
+  if (argc >= 3 && strcmp(argv[1], "-x") == 0)   // the must crash tests
   {
     check(argc-2, argv+2, checker3);
     return 0;
   }
 
-  // setup internationalisation
+  bool fullscreen = false;
+  if (argc >= 2 && strcmp(argv[1], "-f") == 0) fullscreen = true;
+
+  // setup internationalization
+#if defined (__AROS__)
+  amigaLocale(LC_MESSAGES);
+#endif
   setlocale(LC_MESSAGES, "");
   bindtextdomain("pushover", getLocaleDir().c_str());
   bind_textdomain_codeset("pushover", "UTF-8");
@@ -248,12 +305,17 @@ int main(int argc, char * argv[]) {
   const std::string datadir = getDataDir();
   std::string selectedMission;  // the mission that was selected in menu
 
+  // initialize random number generator
+  srandFromTime();
+
   // initialize SDL, graphics, timer, video mode, and level data structure
   SDL_Init(SDL_INIT_TIMER);
   SDL_Init(SDL_INIT_VIDEO);
+  SDL_EnableUNICODE(1);
   atexit(SDL_Quit);
   graphicsN_c gr(datadir);
   screen_c screen(gr);
+  if (fullscreen) screen.toggleFullscreen();
   gr.loadGraphics();
   initText(datadir);
   soundSystem_c::instance()->openSound(datadir);
@@ -261,16 +323,10 @@ int main(int argc, char * argv[]) {
   recorder_c rec;
   ant_c a(l, gr, screen);
   solvedMap_c solved;
+  std::string lwindowLevel = "";  // when it contains a valid levelname, the level is selected when the levelwindow is opened the next time
 
   // prepare the list of levelsets
-  levelsetList_c levelsetList;
-  levelsetList.load(datadir + "/levels");
-  {
-    const std::string userleveldir(getHome() + "/.pushover/levels");
-    struct stat st;
-    if (stat(userleveldir.c_str(), &st) == 0)
-      levelsetList.load(userleveldir);
-  }
+  levelsetList_c * levelsetList = loadAllLevels(datadir, "");
 
   states_e currentState = ST_INIT, nextState = ST_INIT;
 
@@ -281,14 +337,14 @@ int main(int argc, char * argv[]) {
     try {
       rec.load(argv[2]);
       selectedMission = rec.getLevelsetName();
-      levelsetList.getLevelset(selectedMission).loadLevel(l, rec.getLevelName());
+      levelsetList->getLevelset(selectedMission).loadLevel(l, rec.getLevelName(), "");
       a.initForLevel();
       soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
       nextState = ST_REPLAY;
     }
     catch  (...) {
-      nextState = ST_MAIN;
+      nextState = ST_PROFILE_INIT;
     }
   }
   else if (argc == 3)
@@ -301,14 +357,14 @@ int main(int argc, char * argv[]) {
     // it fails fall back to main menu
     try {
 
-      levelsetList.getLevelset(selectedMission).loadLevel(l, levelName);
+      levelsetList->getLevelset(selectedMission).loadLevel(l, levelName, "");
       a.initForLevel();
       soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
       nextState = ST_PREPLAY;
     }
     catch (...) {
-      nextState = ST_MAIN;
+      nextState = ST_PROFILE_INIT;
     }
   }
   else if (argc == 2)
@@ -319,7 +375,7 @@ int main(int argc, char * argv[]) {
     if (file)
     {
       textsections_c sections(file, true);
-      l.load(sections);
+      l.load(sections, "");
       selectedMission = "Original";            // TODO we need to find out which levelset this file belongs to
       soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
 
@@ -327,18 +383,18 @@ int main(int argc, char * argv[]) {
     }
     else
     {
-      nextState = ST_MAIN;
+      nextState = ST_PROFILE_INIT;
       screen.markAllDirty();
     }
   }
   else
   {
-    nextState = ST_MAIN;
+    nextState = ST_PROFILE_INIT;
     screen.markAllDirty();
   }
 
 
-  // not we have initialized, lets get playing in the main state mashine
+  // not we have initialized, lets get playing in the main state
   bool exitProgram = false;
   Uint32 ticks = SDL_GetTicks();
 
@@ -346,8 +402,6 @@ int main(int argc, char * argv[]) {
   unsigned int failReason = 0;
   unsigned int failDelay = 0; // a counter to delay the fail window a bit after failing
 
-  screen.toggleFullscreen();
-  
   if (nextState == ST_MAIN)
   {
     soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
@@ -375,9 +429,15 @@ int main(int argc, char * argv[]) {
             case ST_LEVELSET:
             case ST_LEVEL:
             case ST_CONFIG:
+            case ST_LEVELCONF:
             case ST_SOLVED:
             case ST_FAILED:
             case ST_ABOUT:
+            case ST_PROFILE:
+            case ST_PROFILE_INIT:
+            case ST_PROFILE_IN:
+            case ST_PROFILE_DEL:
+            case ST_TIMEOUT:
               delete window;
               window = 0;
               l.drawDominos();
@@ -391,7 +451,6 @@ int main(int argc, char * argv[]) {
             case ST_PLAY:
             case ST_FAILDELAY:
             case ST_EXIT:
-            case ST_CONFIGTOG:
               break;
 
           }
@@ -400,13 +459,24 @@ int main(int argc, char * argv[]) {
           switch (nextState) {
 
             case ST_MAIN:     window = getMainWindow(screen, gr); break;
-            case ST_LEVELSET: window = getMissionWindow(levelsetList, screen, gr); break;
-            case ST_LEVEL:    window = getLevelWindow(levelsetList.getLevelset(selectedMission), solved, screen, gr); break;
+            case ST_PROFILE_INIT:
+            case ST_PROFILE:  window = getProfileWindow(solved, screen, gr); break;
+            case ST_PROFILE_IN: window = getProfileInputWindow(screen, gr); break;
+            case ST_PROFILE_DEL: window = getProfileSelector(solved, screen, gr); break;
+            case ST_LEVELSET: window = getMissionWindow(*levelsetList, screen, gr, selectedMission); break;
             case ST_QUIT:     window = getQuitWindow(screen, gr); break;
+            case ST_LEVELCONF:
             case ST_CONFIG:   window = getConfigWindow(screen, gr); break;
             case ST_SOLVED:   window = getSolvedWindow(screen, gr); break;
             case ST_ABOUT:    window = getAboutWindow(screen, gr); break;
             case ST_FAILED:   window = getFailedWindow(failReason, screen, gr); break;
+            case ST_TIMEOUT:  window = getTimeoutWindow(screen, gr); break;
+
+
+            case ST_LEVEL:
+                window = getLevelWindow(levelsetList->getLevelset(selectedMission), solved, screen, gr, lwindowLevel);
+                lwindowLevel = "";
+                break;
 
             case ST_HELP:
                               {
@@ -429,19 +499,17 @@ int main(int argc, char * argv[]) {
                               break;
 
             case ST_PLAY:
+                              ticks = SDL_GetTicks();    // the flip might have taken some time too long
                               rec.setLevel(selectedMission, l.getName());
                               rec.reset();
                               break;
             case ST_INIT:
             case ST_REPLAY:
             case ST_FAILDELAY:
-            case ST_CONFIGTOG:
                               break;
 
             case ST_EXIT:
                               exitProgram = true;
-			      if (osso_context)
-			        osso_deinitialize(osso_context);
                               break;
 
           }
@@ -453,6 +521,14 @@ int main(int argc, char * argv[]) {
         SDL_Event event;
         if (!SDL_PollEvent(&event)) break;
 
+        // check for quit-event
+        if (event.type == SDL_QUIT)
+        {
+          // exit program, wherever we are
+          nextState = ST_EXIT;
+          break;
+        }
+
         switch (currentState) {
 
           case ST_MAIN:
@@ -469,14 +545,109 @@ int main(int argc, char * argv[]) {
                 {
                   case 0: nextState = ST_LEVELSET; break;// select level set
                   case 1: nextState = ST_CONFIG; break;  // open config menu
-                  case 2: nextState = ST_ABOUT; break;   // about window
-                  case 3: nextState = ST_EXIT; break;    // exit program
+                  case 2: nextState = ST_PROFILE; break; // open profile selector
+                  case 3: nextState = ST_ABOUT; break;   // about window
+                  case 4: nextState = ST_EXIT; break;    // exit program
+                }
+              }
+            }
+            break;
+
+          case ST_PROFILE:
+          case ST_PROFILE_INIT:
+            if (!window) {
+              std::cout << "Oops no window\n";
+              nextState = ST_MAIN;
+            }
+            else
+            {
+              window->handleEvent(event);
+              if (window->isDone())
+              {
+                unsigned int sel = dynamic_cast<listWindow_c*>(window)->getSelection();
+                if (sel < solved.getNumberOfUsers())
+                {
+                  solved.selectUser(sel);
+                  delete levelsetList;
+                  levelsetList = loadAllLevels(datadir, solved.getUserString());
+
+                  nextState = ST_MAIN;
+                }
+                else if (sel == solved.getNumberOfUsers())
+                {
+                  nextState = ST_PROFILE_IN;
+                }
+                else if (sel == solved.getNumberOfUsers()+1)
+                {
+                  nextState = ST_PROFILE_DEL;
+                }
+                else
+                {
+                  nextState = ST_MAIN;
+                }
+              }
+            }
+            break;
+
+          case ST_PROFILE_DEL:
+            if (!window) {
+              std::cout << "Oops no window\n";
+              nextState = ST_MAIN;
+            }
+            else
+            {
+              window->handleEvent(event);
+              if (window->isDone())
+              {
+                size_t s = dynamic_cast<listWindow_c*>(window)->getSelection();
+
+                if (s+1 < solved.getNumberOfUsers())
+                {
+                  // valid selection available
+
+                  // if the currently selected profile is deleted, go to default
+                  if (s+1 == solved.getCurrentUser())
+                  {
+                    solved.selectUser(0);
+                    delete levelsetList;
+                    levelsetList = loadAllLevels(datadir, solved.getUserString());
+                  }
+
+                  solved.deleteUser(s+1);
+                }
+
+                nextState = ST_PROFILE;
+              }
+            }
+            break;
+
+          case ST_PROFILE_IN:
+            if (!window) {
+              std::cout << "Oops no window\n";
+              nextState = ST_MAIN;
+            }
+            else
+            {
+              window->handleEvent(event);
+              if (window->isDone())
+              {
+                if (!dynamic_cast<InputWindow_c*>(window)->hasEscaped())
+                {
+                  solved.addUser(dynamic_cast<InputWindow_c*>(window)->getText());
+                  delete levelsetList;
+                  levelsetList = loadAllLevels(datadir, solved.getUserString());
+                  nextState = ST_MAIN;
+                }
+                else
+                {
+                  nextState = ST_PROFILE;
                 }
               }
             }
             break;
 
           case ST_CONFIG:
+          case ST_LEVELCONF:
             if (!window) {
               std::cout << "Oops no window\n";
               nextState = ST_MAIN;
@@ -491,15 +662,26 @@ int main(int argc, char * argv[]) {
                   case 0:   // toggle full screen
                     screen.toggleFullscreen();
                     screen.markAllDirty();
-                    nextState = ST_CONFIGTOG;
+                    window->resetWindow();
                     break;
 
                   case 1:  // toggle sound effects
-                    soundSystem_c::instance()->toggleOnOff();
-                    nextState = ST_CONFIGTOG;
+                    soundSystem_c::instance()->toggleSound();
+                    window->resetWindow();
                     break;
 
-                  default: nextState = ST_MAIN; break;  // back to main menu
+                  case 2:  // toggle music
+                    soundSystem_c::instance()->toggleMusic();
+                    window->resetWindow();
+                    break;
+
+                  default:
+                    if (currentState == ST_CONFIG)
+                      nextState = ST_MAIN; // back to main menu
+                    else
+                      nextState = ST_QUIT; // back to level quit dialog
+
+                    break;
                 }
               }
             }
@@ -516,12 +698,13 @@ int main(int argc, char * argv[]) {
               if (window->isDone())
               {
                 unsigned int sel = dynamic_cast<listWindow_c*>(window)->getSelection();
-                if (sel >= levelsetList.getLevelsetNames().size())
+                if (sel >= levelsetList->getLevelsetNames().size())
                   nextState = ST_MAIN;
                 else
                 {
                   nextState = ST_LEVEL;
-                  selectedMission = levelsetList.getLevelsetNames()[sel];
+                  lwindowLevel = "";
+                  selectedMission = levelsetList->getLevelsetNames()[sel];
                 }
               }
             }
@@ -538,14 +721,14 @@ int main(int argc, char * argv[]) {
               if (window->isDone())
               {
                 unsigned int sel = dynamic_cast<listWindow_c*>(window)->getSelection();
-                levelset_c ls = levelsetList.getLevelset(selectedMission);
+                levelset_c ls = levelsetList->getLevelset(selectedMission);
 
                 if (sel >= ls.getLevelNames().size())
                   nextState = ST_LEVELSET;
                 else
                 {
                   nextState = ST_PREPLAY;
-                  ls.loadLevel(l, ls.getLevelNames()[sel]);
+                  ls.loadLevel(l, ls.getLevelNames()[sel], solved.getUserString());
                   soundSystem_c::instance()->playMusic(datadir+"/themes/"+l.getTheme()+".ogg");
                   a.initForLevel();
                 }
@@ -554,21 +737,21 @@ int main(int argc, char * argv[]) {
             break;
 
           case ST_PLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_QUIT;
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_h)     nextState = ST_HELP;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_QUIT;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_F1)     nextState = ST_HELP;
             if (event.type == SDL_KEYDOWN && event.key.keysym.sym == 'r')         rec.save("man");
             break;
 
           case ST_PREPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_PLAY;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_PLAY;
             break;
 
           case ST_PREREPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q) nextState = ST_REPLAY;
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE) nextState = ST_REPLAY;
             break;
 
           case ST_REPLAY:
-            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_q)
+            if (event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE)
             {
               rec.truncate();
               nextState = ST_PLAY;
@@ -613,21 +796,26 @@ int main(int argc, char * argv[]) {
               {
                 switch(dynamic_cast<listWindow_c*>(window)->getSelection())
                 {
-                  case 2:
+                  case 3:
                     nextState = ST_LEVEL;
                     soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
+                    lwindowLevel = l.getName();
                     break;    // return to level list
                   case 1:
                           {       // restart level
-                            nextState = ST_PREPLAY;
-                            levelset_c ls = levelsetList.getLevelset(selectedMission);
-                            ls.loadLevel(l, l.getName());
+                            nextState = ST_PLAY;
+                            levelset_c ls = levelsetList->getLevelset(selectedMission);
+                            ls.loadLevel(l, l.getName(), solved.getUserString());
                             a.initForLevel();
                           }
                           break;
+                  case 2:  // configuration
+                    nextState = ST_LEVELCONF;
+                    break;
                   default:
                   case 0:
-                          nextState = ST_PLAY; break;    // return to play
+                    nextState = ST_PLAY;
+                    break;    // return to play
                 }
               }
             }
@@ -647,6 +835,7 @@ int main(int argc, char * argv[]) {
                 {
                   case 0:
                     nextState = ST_LEVEL;
+                    lwindowLevel = "";
                     break; // select next level to play
                 }
               }
@@ -667,9 +856,8 @@ int main(int argc, char * argv[]) {
                 {
                   case 0:                            // try again
                     {
-                      nextState = ST_PREPLAY;
                       // find the current level
-                      levelset_c ls = levelsetList.getLevelset(selectedMission);
+                      levelset_c ls = levelsetList->getLevelset(selectedMission);
 
                       std::vector<std::string> levels = ls.getLevelNames();
 
@@ -679,19 +867,26 @@ int main(int argc, char * argv[]) {
                       {
                         if (levels[i] == l.getName())
                         {
-                          ls.loadLevel(l, levels[i]);
+                          ls.loadLevel(l, levels[i], solved.getUserString());
                           a.initForLevel();
                           foundLevel = true;
                           break;
                         }
                       }
 
-                      if (!foundLevel) nextState = ST_MAIN;
-                      else nextState = ST_PREPLAY;
+                      if (!foundLevel)
+                      {
+                        nextState = ST_MAIN;
+                      }
+                      else
+                      {
+                        nextState = ST_PLAY;
+                      }
                     }
                     break;
                   case 1:
                     nextState = ST_LEVEL;
+                    lwindowLevel = l.getName();
                     soundSystem_c::instance()->playMusic(datadir+"/themes/option.ogg");
                     break;  // back to level list
                 }
@@ -699,8 +894,58 @@ int main(int argc, char * argv[]) {
             }
             break;
 
+          case ST_TIMEOUT:
+            if (!window) {
+              std::cout << "Oops no window\n";
+              nextState = ST_MAIN;
+            }
+            else
+            {
+              window->handleEvent(event);
+              if (window->isDone())
+              {
+                switch(dynamic_cast<listWindow_c*>(window)->getSelection())
+                {
+                  case 0:                            // try again
+                    {
+                      // find the current level
+                      levelset_c ls = levelsetList->getLevelset(selectedMission);
+
+                      std::vector<std::string> levels = ls.getLevelNames();
+
+                      bool foundLevel = false;
+
+                      for (unsigned int i = 0; i < levels.size(); i++)
+                      {
+                        if (levels[i] == l.getName())
+                        {
+                          ls.loadLevel(l, levels[i], solved.getUserString());
+                          a.initForLevel();
+                          foundLevel = true;
+                          break;
+                        }
+                      }
+
+                      if (!foundLevel)
+                      {
+                        nextState = ST_MAIN;
+                      }
+                      else
+                      {
+                        nextState = ST_PLAY;
+                      }
+                    }
+                    break;
+                  case 1:                            // continue to next level
+                    nextState = ST_LEVEL;
+                    lwindowLevel = "";
+                    break; // select next level to play
+                }
+              }
+            }
+            break;
+
           case ST_INIT:
-          case ST_CONFIGTOG:
           case ST_FAILDELAY:
           case ST_EXIT:
             break;
@@ -765,7 +1010,9 @@ int main(int argc, char * argv[]) {
               case 0:
                 break;
               case 2:
-                nextState = ST_FAILED;
+                rec.save("time");
+                solved.addLevel(l.getChecksumNoTime());
+                nextState = ST_TIMEOUT;
                 break;
               default:
                 failDelay = 36;
@@ -775,17 +1022,19 @@ int main(int argc, char * argv[]) {
           }
           break;
 
-        case ST_CONFIGTOG:
-          nextState = ST_CONFIG;
-          break;
-
         case ST_INIT:
         case ST_MAIN:
+        case ST_PROFILE:
+        case ST_PROFILE_INIT:
+        case ST_PROFILE_IN:
+        case ST_PROFILE_DEL:
         case ST_CONFIG:
+        case ST_LEVELCONF:
         case ST_LEVELSET:
         case ST_LEVEL:
         case ST_SOLVED:
         case ST_FAILED:
+        case ST_TIMEOUT:
         case ST_HELP:
         case ST_QUIT:
         case ST_EXIT:
