@@ -1,6 +1,34 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #include "pngloader.h"
 
+#if defined(__AROS__)
+#include <aros/cpu.h>
+#if AROS_BIG_ENDIAN
+#define WORDS_BIGENDIAN
+#endif
+#else
 #include "../config.h"
+#endif
 
 #include <iostream>
 
@@ -50,9 +78,9 @@ pngLoader_c::pngLoader_c(std::string fname) {
   png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth,
       &color_type, &interlace_type, NULL, NULL);
 
-  if (color_type != PNG_COLOR_TYPE_RGB_ALPHA && color_type != PNG_COLOR_TYPE_PALETTE ||
+  if ((color_type != PNG_COLOR_TYPE_RGB_ALPHA && color_type != PNG_COLOR_TYPE_PALETTE) ||
       interlace_type != PNG_INTERLACE_NONE ||
-      color_type == PNG_COLOR_TYPE_RGB_ALPHA && bit_depth != 8) {
+      (color_type == PNG_COLOR_TYPE_RGB_ALPHA && bit_depth != 8)) {
 
     if (color_type != PNG_COLOR_TYPE_RGB_ALPHA && color_type != PNG_COLOR_TYPE_PALETTE) printf("color type not ok\n");
     if (interlace_type != PNG_INTERLACE_NONE) printf("interlace mode not right\n");
