@@ -1,18 +1,37 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #include "screen.h"
 
 #include "graphics.h"
 
-#include "libintl.h"
-
 #include <SDL/SDL_ttf.h>
 
 #include <iostream>
 #include <vector>
 
 screen_c::screen_c(const graphics_c & g) :
-  gr(g), animationState(0)
+  animationState(0), blockX(g.blockX()), blockY(g.blockY())
 {
-  video = SDL_SetVideoMode(gr.resolutionX(), gr.resolutionY(), 24, 0);
+  video = SDL_SetVideoMode(g.resolutionX(), g.resolutionY(), 24, 0);
   SDL_WM_SetCaption("Pushover", "Pushover");
 }
 
@@ -98,7 +117,7 @@ void surface_c::copy(surface_c & s, int x, int y, int w, int h) {
 
 void surface_c::gradient(int x, int y, int w, int h) {
 
-  // make sure we only work with valid pixels and
+  // make sure we only work with valid pixels
   if (y+h >= video->h) h = video->h-y;
 
   for (int i = 0; i < h; i++)
@@ -124,6 +143,7 @@ void surface_c::gradient(int x, int y, int w, int h) {
 void screen_c::flipComplete(void)
 {
   SDL_Flip(video);
+  animationState = 0;
 }
 
 void screen_c::flipDirty(void)
@@ -144,21 +164,22 @@ void screen_c::flipDirty(void)
       }
       else if (rowStart != -1)
       {
-        rects[numrects].y = gr.blockY()*y;
-        rects[numrects].x = gr.blockX()*rowStart;
+        rects[numrects].y = blockY*y;
+        rects[numrects].x = blockX*rowStart;
 
         if (y == 12)
-          rects[numrects].h = gr.blockY()/2;
+          rects[numrects].h = blockY/2;
         else
-          rects[numrects].h = gr.blockY();
+          rects[numrects].h = blockY;
 
-        rects[numrects].w = gr.blockX()*(x-rowStart);
+        rects[numrects].w = blockX*(x-rowStart);
         numrects++;
         rowStart = -1;
       }
     }
   }
   SDL_UpdateRects(video, numrects, rects);
+  animationState = 0;
 }
 
 static int clip(int v) {
@@ -168,21 +189,29 @@ static int clip(int v) {
 }
 
 // a list of functions that return value between 0 and 255, depending on x and y
-static int f1(int x, int y, int a) { return clip(y*256/3 - 1024 + a*((1024+256)/32)); }
-static int f2(int x, int y, int a) { return clip(y*x*256/3/20 - 1024 + a*((1024+256)/32)); }
-static int f3(int x, int y, int a) { return clip(((2*y-12)*(2*y-12)+(2*x-19)*(2*x-19))*256/110 - 1024 + a*((1024+256)/32)); }
+static int f1(int x, int y, int a) { return clip((13-y)*64 - 13*64 + a*((13*64+256)/64)); }
+static int f2(int x, int y, int a) { return clip(x*32 - 20*32 + a*((32*20+256)/64)); }
+static int f3(int x, int y, int a) { return clip(((2*y-12)*(2*y-12)+(2*x-19)*(2*x-19))*256/127 - 1024 + a*((1024+256)/64)); }
 
-static void u1(SDL_Surface * video, int x, int y, int f, int blx, int bly) {
+static SDL_Rect rects[13*20*255];
+static int count;
+
+static void u1(int x, int y, int f0, int f, int blx, int bly) {
   int by = bly*y;
   int bx = blx*x;
   int bw = f*blx/256;
   int bh = (y == 12) ? bly/2 : bly;
 
-  if (bw > 0)
-    SDL_UpdateRect(video, bx, by, bw, bh);
+  if (bw > 0){
+    rects[count].x=bx;
+    rects[count].y=by;
+    rects[count].w=bw;
+    rects[count].h=bh;
+    count++;
+  }
 }
 
-static void u2(SDL_Surface * video, int x, int y, int f, int blx, int bly) {
+static void u2(int x, int y, int f0, int f, int blx, int bly) {
   int by = bly*y;
   int bx = blx*x;
   int bw = blx;
@@ -190,11 +219,16 @@ static void u2(SDL_Surface * video, int x, int y, int f, int blx, int bly) {
 
   bh = bh*f/256;
 
-  if (bh > 0)
-    SDL_UpdateRect(video, bx, by, bw, bh);
+  if (bh > 0) {
+    rects[count].x=bx;
+    rects[count].y=by;
+    rects[count].w=bw;
+    rects[count].h=bh;
+    count++;
+  }
 }
 
-static void u3(SDL_Surface * video, int x, int y, int f, int blx, int bly) {
+static void u3(int x, int y, int f0, int f, int blx, int bly) {
   int by = bly*y + bly/2 - bly/2*f/256;
   int bx = blx*x + blx/2 - blx/2*f/256;
   int bw = blx;
@@ -203,50 +237,91 @@ static void u3(SDL_Surface * video, int x, int y, int f, int blx, int bly) {
   bh = bh*f/256;
   bw = bw*f/256;
 
-  if (bh > 0)
-    SDL_UpdateRect(video, bx, by, bw, bh);
+  if (bh > 0) {
+    rects[count].x=bx;
+    rects[count].y=by;
+    rects[count].w=bw;
+    rects[count].h=bh;
+    count++;
+  }
+}
+
+static void u4(int x, int y, int f0, int f, int blx, int bly) {
+
+  uint8_t rnd = (x+20*y) % 64;
+
+  f0 /= 4;
+  f /= 4;
+
+  if (f0 == f) return;
+
+  for (int i = 0; i <= f; i++)
+  {
+    if (i >= f0)
+    {
+      int xp = rnd % 8;
+      int yp = rnd / 8;
+
+      int by = bly*y+yp*bly/8;
+      int bx = blx*x+xp*blx/8;
+      int bw = blx/8;
+      int bh = bly/8;
+
+      if (bh > 0 && by < 600) {
+        rects[count].x=bx;
+        rects[count].y=by;
+        rects[count].w=bw;
+        rects[count].h=bh;
+        count++;
+      }
+    }
+
+    rnd = (21*rnd+11) % 64;
+  }
 }
 
 bool screen_c::flipAnimate(void)
 {
   static int (*f)(int, int, int);
-  static void (*u)(SDL_Surface *, int, int, int, int, int);
+  static void (*u)(int, int, int, int, int, int);
 
   if (animationState == 0) {
     switch (rand()%3) {
-      case 0: f = f1; break;
-      case 1: f = f2; break;
-      case 2: f = f3; break;
+      case 0: f = f1; break;  // up down sweep
+      case 1: f = f2; break;  // right to left sewep
+      case 2: f = f3; break;  // circular outside in
       default: f = f3; break;
     }
-    switch (rand()%3) {
-      case 0: u = u1; break;
-      case 1: u = u2; break;
-      case 2: u = u3; break;
+    switch (rand()%4) {
+      case 0: u = u1; break;  // left --> right update
+      case 1: u = u2; break;  // up --> down update
+      case 2: u = u3; break;  // inside out update
+      case 3: u = u4; break;  // diffuse update
       default: u = u3; break;
     }
   }
 
+
   animationState++;
+  count=0;
 
   for (int y = 0; y < 13; y++)
   {
     for (int x = 0; x < 20; x++)
     {
-      if (isDirty(x, y))
-      {
-        int valNew = f(x, y, animationState);
-        int valOld = f(x, y, animationState-1);
+      int valNew = f(x, y, animationState);
+      int valOld = f(x, y, animationState-1);
 
-        if (valNew != valOld)
-        {
-          u(video, x, y, valNew, gr.blockX(), gr.blockY());
-        }
+      if (valNew != valOld)
+      {
+        u(x, y, valOld, valNew, blockX, blockY);
       }
     }
   }
 
-  if (animationState == 32)
+  SDL_UpdateRects(video, count, rects);
+
+  if (animationState == 64)
   {
     animationState = 0;
     return true;
@@ -263,9 +338,6 @@ SDL_Surface * surface_c::getIdentical(void) const {
     return SDL_CreateRGBSurface(0, video->w, video->h, 32, video->format->Rmask, video->format->Gmask, video->format->Bmask, 0);
 }
 
-
-pixelSurface_c::pixelSurface_c(const surface_c & pre) : surface_c(pre.getIdentical()) { }
-
 static std::vector<TTF_Font * > fonts;
 
 
@@ -280,7 +352,7 @@ void initText(std::string dir) {
 
   TTF_Font * ft;
 
-  ft = TTF_OpenFont(fname.c_str(), 20);
+  ft = TTF_OpenFont(fname.c_str(), 18);
   if (!ft) {
     std::cout << "Can't open Font file: " << fname << std::endl;
     exit(1);
@@ -336,9 +408,25 @@ static std::vector<std::string> split(const std::string & text, char splitter)
   return res;
 }
 
-unsigned int surface_c::renderText(const fontParams_s * par, const std::string & t) {
+unsigned int surface_c::renderText(const fontParams_s * par, const std::string & t)
+{
+  // make some safety checks, empty strings are not output
+  bool onlySpace = true;
+  for (size_t i = 0; i < t.length(); i++)
+    if (t[i] != ' ')
+    {
+      onlySpace = false;
+      break;
+    }
 
-  std::vector<std::string> words = split(gettext(t.c_str()), ' ');
+  if (   (t.length() == 0)
+      || (onlySpace)
+     )
+  {
+    return 1;
+  }
+
+  std::vector<std::string> words = split(t.c_str(), ' ');;
 
   int ypos = par->box.y;
 
@@ -382,16 +470,27 @@ unsigned int surface_c::renderText(const fontParams_s * par, const std::string &
       r.h = vv->h;
       r.y = ypos;
 
-      if (par->shadow)
+      if (par->shadow == 1)
       {
         int sa = 1;
         if (par->font == FNT_BIG) sa = 2;
 
         r.x-=sa; r.y-=sa; SDL_BlitSurface(vb, 0, video, &r);
-        r.x+=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.y+=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.x-=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.y-=sa; r.x+=sa;
+        r.x+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x+=sa;
+      }
+      else if (par->shadow == 2)
+      {
+        int sa = 1;
+
+        r.x+=sa; r.y+=sa; SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa; r.y-=sa;
       }
       SDL_BlitSurface(vv, 0, video, &r);
     }
@@ -405,16 +504,27 @@ unsigned int surface_c::renderText(const fontParams_s * par, const std::string &
       r.w = vv->w;
       r.h = vv->h;
 
-      if (par->shadow)
+      if (par->shadow == 1)
       {
         int sa = 1;
         if (par->font == FNT_BIG) sa = 2;
 
         r.x-=sa; r.y-=sa; SDL_BlitSurface(vb, 0, video, &r);
-        r.x+=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.y+=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.x-=2*sa;        SDL_BlitSurface(vb, 0, video, &r);
-        r.y-=sa; r.x+=sa;
+        r.x+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y+=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.y-=sa;          SDL_BlitSurface(vb, 0, video, &r);
+        r.x+=sa;
+      }
+      else if (par->shadow == 2)
+      {
+        int sa = 1;
+
+        r.x+=sa; r.y+=sa; SDL_BlitSurface(vb, 0, video, &r);
+        r.x-=sa; r.y-=sa;
       }
       SDL_BlitSurface(vv, 0, video, &r);
     }
@@ -436,15 +546,15 @@ unsigned int getFontHeight(unsigned int font) {
 }
 
 unsigned int getTextWidth(unsigned int font, const std::string & t) {
-  int w;
-  TTF_SizeUTF8(fonts[font], gettext(t.c_str()), &w, 0);
+  int w = 0;
+  TTF_SizeUTF8(fonts[font], t.c_str(), &w, 0);
 
   return w;
 }
 
 unsigned int getTextHeight(const fontParams_s * par, const std::string & t) {
 
-  std::vector<std::string> words = split(gettext(t.c_str()), ' ');
+  std::vector<std::string> words = split(t.c_str(), ' ');
 
   unsigned int word = 0;
   int height = 0;
