@@ -1,3 +1,24 @@
+/* Pushover
+ *
+ * Pushover is the legal property of its developers, whose
+ * names are listed in the COPYRIGHT file, which is included
+ * within the source distribution.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA
+ */
+
 #ifndef __SCREEN_H__
 #define __SCREEN_H__
 
@@ -5,7 +26,15 @@
 
 #include <string>
 
-/* this is the display class, there is only one instance fo this class and it contains
+#ifdef __AROS__
+extern "C" {
+#include "libintl.h"
+}
+#else
+#include "libintl.h"
+#endif
+
+/* this is the display class, there is only one instance for this class and it contains
  * the complete screen
  * the background is always a level.
  * In front of that can be stacked windows
@@ -13,7 +42,7 @@
 
 class graphics_c;
 
-/* the 3 vavailable fonts, they are a fixed size and are supposed to contain
+/* the 3 available fonts, they are a fixed size and are supposed to contain
  * all the required letters
  */
 enum {
@@ -34,7 +63,7 @@ typedef struct {
   SDL_Rect box;
   int alignment;
   SDL_Color color;
-  bool shadow;
+  uint8_t shadow;
 } fontParams_s;
 
 
@@ -96,28 +125,20 @@ class surface_c {
     // return value is the number of lines that were output
     unsigned int renderText(const fontParams_s * par, const std::string & t);
 
-    // apply the gradient within the givne area
+    // apply the gradient within the given area
     void gradient(int x, int y, int w, int h);
 };
 
-class pixelSurface_c : public surface_c {
-
-  public:
-
-    // creates a surface of ther same size and same format at the given surface
-    pixelSurface_c(const surface_c & pre);
-};
-
 class screen_c : public surface_c {
 
   private:
 
-    const graphics_c & gr;
     int animationState;
+    uint16_t blockX, blockY;
 
   public:
 
-    // constructor, does nothing for the time beeing
+    // constructor, does nothing for the time being
     screen_c(const graphics_c & gr);
     ~screen_c(void);
 
@@ -128,8 +149,8 @@ class screen_c : public surface_c {
     void toggleFullscreen(void);
 };
 
-// used for internationalisation to mark strings to translate
-#define _(x) x
+// used for internationalization to mark strings to translate
+#define _(x) gettext(x)
 
 #endif
 
